@@ -1,6 +1,6 @@
 //*****************************************************************************
 //
-//  SPDX-FileCopyrightText: Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+//  SPDX-FileCopyrightText: Copyright (c) 2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 //  SPDX-License-Identifier: MIT
 //
 //  Permission is hereby granted, free of charge, to any person obtaining a
@@ -111,7 +111,7 @@ void updateColorFormatForDisplayIdExtnTimings(NVT_EDID_INFO *pInfo,
     nvt_assert((timingIdx) <= COUNT(pInfo->timing));
 
     if ((pInfo->input.u.digital.video_interface == NVT_EDID_DIGITAL_VIDEO_INTERFACE_STANDARD_HDMI_A_SUPPORTED ||
-         pInfo->input.u.digital.video_interface == NVT_EDID_DIGITAL_VIDEO_INTERFACE_STANDARD_HDMI_B_SUPPORTED || 
+         pInfo->input.u.digital.video_interface == NVT_EDID_DIGITAL_VIDEO_INTERFACE_STANDARD_HDMI_B_SUPPORTED ||
          pInfo->ext861.valid.H14B_VSDB || pInfo->ext861.valid.H20_HF_VSDB) && pInfo->ext861.revision >= NVT_CEA861_REV_A)
     {
          if (!pInfo->ext_displayid.supported_displayId2_0)
@@ -153,7 +153,7 @@ void updateColorFormatForDisplayIdExtnTimings(NVT_EDID_INFO *pInfo,
                                                         pDisplayIdInfo->u4.display_interface_features.rgb_depth.support_10b,
                                                         pDisplayIdInfo->u4.display_interface_features.rgb_depth.support_12b,
                                                         pDisplayIdInfo->u4.display_interface_features.rgb_depth.support_14b,
-                                                        pDisplayIdInfo->u4.display_interface_features.rgb_depth.support_16b);         
+                                                        pDisplayIdInfo->u4.display_interface_features.rgb_depth.support_16b);
          }
     }
 
@@ -174,7 +174,7 @@ void updateColorFormatForDisplayIdExtnTimings(NVT_EDID_INFO *pInfo,
                                                     pDisplayIdInfo->u4.display_interface.ycbcr422_depth.support_14b,
                                                     pDisplayIdInfo->u4.display_interface.ycbcr422_depth.support_16b);
     }
-    else 
+    else
     {
         // yuv444
         UPDATE_BPC_FOR_COLORFORMAT(pT->etc.yuv444, 0, /* yuv444 does not support 6bpc */
@@ -264,7 +264,7 @@ static NVT_STATUS parseDisplayIdSection(DISPLAYID_SECTION * section,
  * @brief Parses a displayID data block
  * @param block The DisplayID data block to parse
  * @param max_length The indicated total length of the each data block for checking
- * @param pLength return the indicated length of the each data block 
+ * @param pLength return the indicated length of the each data block
  * @param pEdidInfo EDID struct containing DisplayID information and
  *                  the timings or validation purpose if it is NULL
  */
@@ -285,7 +285,7 @@ NVT_STATUS parseDisplayIdBlock(NvU8* pBlock,
         return NVT_STATUS_ERR;
 
     pInfo = pEdidInfo == NULL ? NULL : &pEdidInfo->ext_displayid;
-    
+
     *pLength = hdr->data_bytes + NVT_DISPLAYID_DATABLOCK_HEADER_LEN;
 
     switch (hdr->type)
@@ -377,8 +377,6 @@ static NVT_STATUS parseDisplayIdColorChar(NvU8 * block, NVT_DISPLAYID_INFO *pInf
 
     if ((prim_num + wp_num) * sizeof(DISPLAYID_COLOR_POINT) + 1 != blk->header.data_bytes)
     {
-        // Assert since this error is ignored
-        nvt_assert(0);
         return NVT_STATUS_ERR;
     }
 
@@ -386,9 +384,9 @@ static NVT_STATUS parseDisplayIdColorChar(NvU8 * block, NVT_DISPLAYID_INFO *pInf
 
     for (i = 0; i < prim_num; i++)
     {
-        x_p = (blk->points)[i].color_x_bits_low + 
+        x_p = (blk->points)[i].color_x_bits_low +
             (DRF_VAL(T_DISPLAYID, _COLOR, _POINT_X, (blk->points)[i].color_bits_mid) << 8);
-        y_p = DRF_VAL(T_DISPLAYID, _COLOR, _POINT_Y, (blk->points)[i].color_bits_mid) + 
+        y_p = DRF_VAL(T_DISPLAYID, _COLOR, _POINT_Y, (blk->points)[i].color_bits_mid) +
             ((blk->points)[i].color_y_bits_high << 4);
         pInfo->primaries[i].x = x_p;
         pInfo->primaries[i].y = y_p;
@@ -396,9 +394,9 @@ static NVT_STATUS parseDisplayIdColorChar(NvU8 * block, NVT_DISPLAYID_INFO *pInf
 
     for (j = 0; j < wp_num; j++)
     {
-        x_p = (blk->points)[i].color_x_bits_low + 
+        x_p = (blk->points)[i].color_x_bits_low +
             (DRF_VAL(T_DISPLAYID, _COLOR, _POINT_X, (blk->points)[i].color_bits_mid) << 8);
-        y_p = DRF_VAL(T_DISPLAYID, _COLOR, _POINT_Y, (blk->points)[i].color_bits_mid) + 
+        y_p = DRF_VAL(T_DISPLAYID, _COLOR, _POINT_Y, (blk->points)[i].color_bits_mid) +
             ((blk->points)[i].color_y_bits_high << 4);
         pInfo->white_points[pInfo->total_primaries + j].x = x_p;
         pInfo->white_points[pInfo->total_primaries + j].y = y_p;
@@ -416,8 +414,6 @@ static NVT_STATUS parseDisplayIdProdIdentityBlock(NvU8 * block, NVT_DISPLAYID_IN
     DISPLAYID_PROD_IDENTIFICATION_BLOCK * blk = (DISPLAYID_PROD_IDENTIFICATION_BLOCK *)block;
     if (blk->header.data_bytes - blk->productid_string_size != NVT_DISPLAYID_PRODUCT_IDENTITY_MIN_LEN)
     {
-        // Assert since this error is ignored
-        nvt_assert(0);
         return NVT_STATUS_ERR;
     }
 
@@ -442,8 +438,6 @@ static NVT_STATUS parseDisplayIdParam(NvU8 * block, NVT_DISPLAYID_INFO *pInfo)
     DISPLAYID_DISPLAY_PARAM_BLOCK * blk = (DISPLAYID_DISPLAY_PARAM_BLOCK *)block;
     if (blk->header.data_bytes != NVT_DISPLAYID_DISPLAY_PARAM_BLOCK_LEN)
     {
-        // Assert since this error is ignored
-        nvt_assert(0);
         return NVT_STATUS_ERR;
     }
 
@@ -479,8 +473,6 @@ static NVT_STATUS parseDisplayIdTiming1(NvU8 * block, NVT_EDID_INFO *pEdidInfo)
     DISPLAYID_TIMING_1_BLOCK * blk = (DISPLAYID_TIMING_1_BLOCK *)block;
     if (blk->header.data_bytes % sizeof(DISPLAYID_TIMING_1_DESCRIPTOR) != 0)
     {
-        // Assert since this error is ignored
-        nvt_assert(0);
         return NVT_STATUS_ERR;
     }
 
@@ -508,12 +500,12 @@ static NVT_STATUS parseDisplayIdTiming1(NvU8 * block, NVT_EDID_INFO *pEdidInfo)
 CODE_SEGMENT(PAGE_DD_CODE)
 static NVT_STATUS parseDisplayIdTiming1Descriptor(DISPLAYID_TIMING_1_DESCRIPTOR * type1, NVT_TIMING *pT)
 {
-    NvU32 totalPixels_in_2_fields;
     if (type1 == NULL || pT == NULL)
         return NVT_STATUS_ERR;
 
     // the pixel clock
     pT->pclk = (NvU32)((type1->pixel_clock_high << 16) + (type1->pixel_clock_mid << 8) + type1->pixel_clock_low_minus_0_01MHz + 1);
+    pT->pclk1khz = (pT->pclk << 3) + (pT->pclk << 1);
 
     // the DisplayID spec does not support border
     pT->HBorder = pT->VBorder = 0;
@@ -569,30 +561,17 @@ static NVT_STATUS parseDisplayIdTiming1Descriptor(DISPLAYID_TIMING_1_DESCRIPTOR
     }
 
     // the refresh rate
-    if (pT->interlaced)
-    {
-        // in interlaced mode, adjust for one extra line in every other frame. pT->VTotal is field based here
-        totalPixels_in_2_fields = (NvU32)pT->HTotal * ((NvU32)pT->VTotal * 2 + 1);
-        // calculate the field rate in interlaced mode
-        pT->etc.rr = (NvU16)axb_div_c(pT->pclk * 2, 10000, totalPixels_in_2_fields);
-        pT->etc.rrx1k = axb_div_c(pT->pclk * 2, 10000000, totalPixels_in_2_fields);
-    }
-    else
-    {
-        // calculate frame rate in progressive mode
-        // in progressive mode filed = frame
-        pT->etc.rr = (NvU16)axb_div_c(pT->pclk, 10000, (NvU32)pT->HTotal * (NvU32)pT->VTotal);
-        pT->etc.rrx1k = axb_div_c(pT->pclk, 10000000, (NvU32)pT->HTotal * (NvU32)pT->VTotal);
-    }
+    pT->etc.rr = NvTiming_CalcRR(pT->pclk1khz, pT->interlaced, pT->HTotal, pT->VTotal);
+    pT->etc.rrx1k = NvTiming_CalcRRx1k(pT->pclk1khz, pT->interlaced, pT->HTotal, pT->VTotal);
     pT->etc.name[39] = '\0';
     pT->etc.rep = 0x1; // bit mask for no pixel repetition
-    
+
     pT->etc.status = NVT_STATUS_DISPLAYID_1;
     // Unlike the PTM in EDID base block, DisplayID type I/II preferred timing does not have  dependency on sequence
     // so we'll just update the preferred flag, not sequence them
     //pT->etc.status = NVT_STATUS_DISPLAYID_1N(1);
     pT->etc.flag |= type1->options.is_preferred_detailed_timing ? NVT_FLAG_DISPLAYID_DTD_PREFERRED_TIMING : 0;
-    
+
     /* Fields currently not used. Uncomment them for future use
     type1->options.stereo_support;
     */
@@ -621,8 +600,6 @@ static NVT_STATUS parseDisplayIdTiming2(NvU8 * block, NVT_EDID_INFO *pEdidInfo)
 
     if (blk->header.data_bytes % sizeof(DISPLAYID_TIMING_2_DESCRIPTOR) != 0)
     {
-        // Assert since this error is ignored
-        nvt_assert(0);
         return NVT_STATUS_ERR;
     }
 
@@ -651,12 +628,12 @@ static NVT_STATUS parseDisplayIdTiming2(NvU8 * block, NVT_EDID_INFO *pEdidInfo)
 CODE_SEGMENT(PAGE_DD_CODE)
 static NVT_STATUS parseDisplayIdTiming2Descriptor(DISPLAYID_TIMING_2_DESCRIPTOR * type2, NVT_TIMING *pT)
 {
-    NvU32 totalPixels_in_2_fields;
     if (type2 == NULL || pT == NULL)
         return NVT_STATUS_ERR;
 
     // the pixel clock
     pT->pclk = (NvU32)((type2->pixel_clock_high << 16) + (type2->pixel_clock_mid << 8) + type2->pixel_clock_low_minus_0_01MHz + 1);
+    pT->pclk1khz = (pT->pclk << 3) + (pT->pclk << 1);
 
     // the DisplayID spec does not support border
     pT->HBorder = pT->VBorder = 0;
@@ -679,32 +656,19 @@ static NVT_STATUS parseDisplayIdTiming2Descriptor(DISPLAYID_TIMING_2_DESCRIPTOR
     pT->interlaced = type2->options.interface_frame_scanning_type;
 
     // the refresh rate
-    if (pT->interlaced)
-    {
-        // in interlaced mode, adjust for one extra line in every other frame. pT->VTotal is field based here
-        totalPixels_in_2_fields = (NvU32)pT->HTotal * ((NvU32)pT->VTotal * 2 + 1);
-        // calculate the field rate in interlaced mode
-        pT->etc.rr = (NvU16)axb_div_c(pT->pclk * 2, 10000, totalPixels_in_2_fields);
-        pT->etc.rrx1k = axb_div_c(pT->pclk * 2, 10000000, totalPixels_in_2_fields);
-    }
-    else
-    {
-        // calculate frame rate in progressive mode
-        // in progressive mode filed = frame
-        pT->etc.rr = (NvU16)axb_div_c(pT->pclk, 10000, (NvU32)pT->HTotal * (NvU32)pT->VTotal);
-        pT->etc.rrx1k = axb_div_c(pT->pclk, 10000000, (NvU32)pT->HTotal * (NvU32)pT->VTotal);
-    }
+    pT->etc.rr = NvTiming_CalcRR(pT->pclk1khz, pT->interlaced, pT->HTotal, pT->VTotal);
+    pT->etc.rrx1k = NvTiming_CalcRRx1k(pT->pclk1khz, pT->interlaced, pT->HTotal, pT->VTotal);
 
     pT->etc.aspect = 0;
     pT->etc.name[39] = '\0';
     pT->etc.rep = 0x1; // Bit mask for no pixel repetition
-    
+
     pT->etc.status = NVT_STATUS_DISPLAYID_2;
     // Unlike the PTM in EDID base block, DisplayID type I/II preferred timing does not have dependency on sequence
     // so we'll just update the preferred flag, not sequence them
     //pT->etc.status = NVT_STATUS_DISPLAYID_1N(1);
     pT->etc.flag |= type2->options.is_preferred_detailed_timing ? NVT_FLAG_DISPLAYID_DTD_PREFERRED_TIMING : 0;
-    
+
     /* Fields currently not used. Uncomment them for future use
     type1->options.stereo_support;
     */
@@ -794,8 +758,6 @@ static NVT_STATUS parseDisplayIdTiming3(NvU8 * block, NVT_EDID_INFO *pEdidInfo)
 
     if (blk->header.data_bytes % sizeof(DISPLAYID_TIMING_3_DESCRIPTOR) != 0)
     {
-        // Assert since this error is ignored
-        nvt_assert(0);
         return NVT_STATUS_ERR;
     }
 
@@ -829,8 +791,6 @@ static NVT_STATUS parseDisplayIdTiming4(NvU8 * block, NVT_EDID_INFO *pEdidInfo)
     DISPLAYID_TIMING_4_BLOCK * blk = (DISPLAYID_TIMING_4_BLOCK *)block;
     if (blk->header.data_bytes < 1 || blk->header.data_bytes > NVT_DISPLAYID_DATABLOCK_MAX_PAYLOAD_LEN)
     {
-        // Assert since this error is ignored
-        nvt_assert(0);
         return NVT_STATUS_ERR;
     }
 
@@ -861,12 +821,12 @@ static NVT_STATUS parseDisplayIdTiming5Descriptor(DISPLAYID_TIMING_5_DESCRIPTOR
 {
     NvU32 width, height, rr;
     NvBool is1000div1001 = NV_FALSE;
-    
+
     // we don't handle stereo type nor custom reduced blanking yet
     //NvU8 stereoType, formula;
     //stereoType = (desc->optns & NVT_DISPLAYID_TIMING_5_STEREO_SUPPORT_MASK);
     //formula = desc->optns & NVT_DISPLAYID_TIMING_5_FORMULA_SUPPORT_MASK;
-    
+
     if (desc->optns & NVT_DISPLAYID_TIMING_5_FRACTIONAL_RR_SUPPORT_MASK)
     {
         is1000div1001 = NV_TRUE;
@@ -874,7 +834,7 @@ static NVT_STATUS parseDisplayIdTiming5Descriptor(DISPLAYID_TIMING_5_DESCRIPTOR
     width = ((desc->horizontal_active_pixels_high << 8) | desc->horizontal_active_pixels_low) + 1;
     height = ((desc->vertical_active_pixels_high << 8) | desc->vertical_active_pixels_low) + 1;
     rr = desc->refresh_rate + 1;
-    return NvTiming_CalcCVT_RB2(width, height, rr, is1000div1001, pT);
+    return NvTiming_CalcCVT_RB2(width, height, rr, is1000div1001, NV_FALSE, pT);
 }
 
 CODE_SEGMENT(PAGE_DD_CODE)
@@ -885,14 +845,12 @@ static NVT_STATUS parseDisplayIdTiming5(NvU8 * block, NVT_EDID_INFO *pEdidInfo)
     DISPLAYID_TIMING_5_BLOCK * blk = (DISPLAYID_TIMING_5_BLOCK *)block;
     if (blk->header.data_bytes < 1 || blk->header.data_bytes > NVT_DISPLAYID_DATABLOCK_MAX_PAYLOAD_LEN)
     {
-        // Assert since this error is ignored
-        nvt_assert(0);
         return NVT_STATUS_ERR;
     }
     for (i = 0; i * sizeof(DISPLAYID_TIMING_5_DESCRIPTOR) < blk->header.data_bytes; i++)
     {
         NVMISC_MEMSET(&newTiming, 0, sizeof(newTiming));
-        
+
         if (parseDisplayIdTiming5Descriptor(blk->descriptors + i, &newTiming) == NVT_STATUS_SUCCESS)
         {
             if (pEdidInfo == NULL) continue;
@@ -918,8 +876,6 @@ static NVT_STATUS parseDisplayIdTimingVesa(NvU8 * block, NVT_EDID_INFO *pEdidInf
     DISPLAYID_TIMING_MODE_BLOCK * blk = (DISPLAYID_TIMING_MODE_BLOCK *)block;
     if (blk->header.data_bytes != DISPLAYID_TIMING_VESA_BLOCK_SIZE)
     {
-        // Assert since this error is ignored
-        nvt_assert(0);
         return NVT_STATUS_ERR;
     }
 
@@ -959,8 +915,6 @@ static NVT_STATUS parseDisplayIdTimingEIA(NvU8 * block, NVT_EDID_INFO *pEdidInfo
     DISPLAYID_TIMING_MODE_BLOCK * blk = (DISPLAYID_TIMING_MODE_BLOCK *)block;
     if (blk->header.data_bytes != DISPLAYID_TIMING_CEA_BLOCK_SIZE)
     {
-        // Assert since this error is ignored
-        nvt_assert(0);
         return NVT_STATUS_ERR;
     }
 
@@ -1003,8 +957,6 @@ static NVT_STATUS parseDisplayIdRangeLimits(NvU8 * block, NVT_DISPLAYID_INFO *pI
 
     if (blk->header.data_bytes != DISPLAYID_RANGE_LIMITS_BLOCK_LEN)
     {
-        // Assert since this error is ignored
-        nvt_assert(0);
         return NVT_STATUS_ERR;
     }
 
@@ -1015,7 +967,7 @@ static NVT_STATUS parseDisplayIdRangeLimits(NvU8 * block, NVT_DISPLAYID_INFO *pI
         blk->vertical_refresh_rate_min > blk->vertical_refresh_rate_max ||
         minPclk > maxPclk)
     {
-        nvt_assert(0 && "wrong range limit");
+        // wrong range limit
         status = NVT_STATUS_ERR;
     }
 
@@ -1023,14 +975,12 @@ static NVT_STATUS parseDisplayIdRangeLimits(NvU8 * block, NVT_DISPLAYID_INFO *pI
 
     if (pInfo->rl_num >= NVT_DISPLAYID_RANGE_LIMITS_MAX_COUNT)
     {
-        // Assert since this error is ignored
-        nvt_assert(0);
         return NVT_STATUS_ERR;
     }
 
     rl = pInfo->range_limits + pInfo->rl_num;
     (pInfo->rl_num)++;
-    
+
     rl->pclk_min = minPclk;
     rl->pclk_max = maxPclk;
 
@@ -1055,8 +1005,6 @@ static NVT_STATUS parseDisplayIdSerialNumber(NvU8 * block, NVT_DISPLAYID_INFO *p
     DISPLAYID_ASCII_STRING_BLOCK * blk = (DISPLAYID_ASCII_STRING_BLOCK *)block;
     if (blk->header.data_bytes > NVT_DISPLAYID_DATABLOCK_MAX_PAYLOAD_LEN)
     {
-        // Assert since this error is ignored
-        nvt_assert(0);
         return NVT_STATUS_ERR;
     }
 
@@ -1074,8 +1022,6 @@ static NVT_STATUS parseDisplayIdAsciiString(NvU8 * block, NVT_DISPLAYID_INFO *pI
     DISPLAYID_ASCII_STRING_BLOCK * blk = (DISPLAYID_ASCII_STRING_BLOCK *)block;
     if (blk->header.data_bytes > NVT_DISPLAYID_DATABLOCK_MAX_PAYLOAD_LEN)
     {
-        // Assert since this error is ignored
-        nvt_assert(0);
         return NVT_STATUS_ERR;
     }
 
@@ -1093,8 +1039,6 @@ static NVT_STATUS parseDisplayIdDeviceData(NvU8 * block, NVT_DISPLAYID_INFO *pIn
     DISPLAYID_DEVICE_DATA_BLOCK * blk = (DISPLAYID_DEVICE_DATA_BLOCK *)block;
     if (blk->header.data_bytes != DISPLAYID_DEVICE_DATA_BLOCK_LEN)
     {
-        // Assert since this error is ignored
-        nvt_assert(0);
         return NVT_STATUS_ERR;
     }
 
@@ -1105,7 +1049,7 @@ static NVT_STATUS parseDisplayIdDeviceData(NvU8 * block, NVT_DISPLAYID_INFO *pIn
     pInfo->device_op_mode = DRF_VAL(T_DISPLAYID, _DEVICE, _OPERATING_MODE, blk->operating_mode);
     pInfo->support_backlight = DRF_VAL(T_DISPLAYID, _DEVICE, _BACKLIGHT, blk->operating_mode);
     pInfo->support_intensity = DRF_VAL(T_DISPLAYID, _DEVICE, _INTENSITY, blk->operating_mode);
-    
+
     pInfo->horiz_pixel_count = blk->horizontal_pixel_count;
     pInfo->vert_pixel_count = blk->vertical_pixel_count;
 
@@ -1131,8 +1075,6 @@ static NVT_STATUS parseDisplayIdInterfacePower(NvU8 * block, NVT_DISPLAYID_INFO
     DISPLAYID_INTERFACE_POWER_BLOCK * blk = (DISPLAYID_INTERFACE_POWER_BLOCK *)block;
     if (blk->header.data_bytes != DISPLAYID_INTERFACE_POWER_BLOCK_LEN)
     {
-        // Assert since this error is ignored
-        nvt_assert(0);
         return NVT_STATUS_ERR;
     }
 
@@ -1169,8 +1111,6 @@ static NVT_STATUS parseDisplayIdDisplayInterface(NvU8 * block, NVT_DISPLAYID_INF
     DISPLAYID_INTERFACE_DATA_BLOCK * blk = (DISPLAYID_INTERFACE_DATA_BLOCK *)block;
     if (blk->header.data_bytes != DISPLAYID_INTERFACE_DATA_BLOCK_LEN)
     {
-        // Assert since this error is ignored
-        nvt_assert(0);
         return NVT_STATUS_ERR;
     }
 
@@ -1205,7 +1145,7 @@ static NVT_STATUS parseDisplayIdDisplayInterface(NvU8 * block, NVT_DISPLAYID_INF
     // Content Protection
     pInfo->u4.display_interface.content_protection = DRF_VAL(T_DISPLAYID, _INTERFACE, _CONTENT, blk->content_protection);
     pInfo->u4.display_interface.content_protection_version = blk->content_protection_version;
-    
+
     // Spread
     pInfo->u4.display_interface.spread_spectrum = DRF_VAL(T_DISPLAYID, _INTERFACE, _SPREAD_TYPE, blk->spread);
     pInfo->u4.display_interface.spread_percent = DRF_VAL(T_DISPLAYID, _INTERFACE, _SPREAD_PER, blk->spread);
@@ -1242,8 +1182,6 @@ static NVT_STATUS parseDisplayIdStereo(NvU8 * block, NVT_DISPLAYID_INFO *pInfo)
     DISPLAYID_STEREO_INTERFACE_METHOD_BLOCK * blk = (DISPLAYID_STEREO_INTERFACE_METHOD_BLOCK *)block;
     if (blk->header.data_bytes > NVT_DISPLAYID_DATABLOCK_MAX_PAYLOAD_LEN)
     {
-        // Assert since this error is ignored
-        nvt_assert(0);
         return NVT_STATUS_ERR;
     }
 
@@ -1274,11 +1212,9 @@ static NVT_STATUS parseDisplayIdStereo(NvU8 * block, NVT_DISPLAYID_INFO *pInfo)
         case NVT_DISPLAYID_STEREO_PROPRIETARY:
             break;
         default:
-            // Assert since this error is ignored
-            nvt_assert(0);
             return NVT_STATUS_ERR;
     }
-    
+
     return NVT_STATUS_SUCCESS;
 }
 
@@ -1288,8 +1224,6 @@ static NVT_STATUS parseDisplayIdTiledDisplay(NvU8 * block, NVT_DISPLAYID_INFO *p
     DISPLAYID_TILED_DISPLAY_BLOCK * blk = (DISPLAYID_TILED_DISPLAY_BLOCK *)block;
     if (blk->header.data_bytes > NVT_DISPLAYID_DATABLOCK_MAX_PAYLOAD_LEN)
     {
-        // Assert since this error is ignored
-        nvt_assert(0);
         return NVT_STATUS_ERR;
     }
 
@@ -1322,7 +1256,7 @@ static NVT_STATUS parseDisplayIdTiledDisplay(NvU8 * block, NVT_DISPLAYID_INFO *p
     pInfo->bezel_info.left                  = (blk->bezel_info.left * blk->bezel_info.pixel_density) / 10;
 
     pInfo->tile_topology_id.vendor_id       = (blk->topology_id.vendor_id[2] << 16) |
-                                              (blk->topology_id.vendor_id[1] << 8 ) | 
+                                              (blk->topology_id.vendor_id[1] << 8 ) |
                                               blk->topology_id.vendor_id[0];
 
     pInfo->tile_topology_id.product_id      = (blk->topology_id.product_id[1] << 8) | blk->topology_id.product_id[0];
@@ -1342,15 +1276,13 @@ static NVT_STATUS parseDisplayIdCtaData(NvU8 * block, NVT_EDID_INFO *pInfo)
     NVT_EDID_CEA861_INFO *p861info;
     if (blk->data_bytes > NVT_DISPLAYID_DATABLOCK_MAX_PAYLOAD_LEN)
     {
-        // Assert since this error is ignored
-        nvt_assert(0);
         return NVT_STATUS_ERR;
     }
 
     if (pInfo == NULL) return NVT_STATUS_SUCCESS;
 
     p861info = &pInfo->ext861;
-    
+
     pInfo->ext_displayid.cea_data_block_present = 1;
     p861info->revision = blk->revision;
 
@@ -1365,16 +1297,19 @@ static NVT_STATUS parseDisplayIdCtaData(NvU8 * block, NVT_EDID_INFO *pInfo)
     parseCta861HfScdb(p861info, pInfo, FROM_DISPLAYID_13_DATA_BLOCK);
 
     //parse HDR related information from the HDR static metadata data block
-    parseCea861HdrStaticMetadataDataBlock(p861info, pInfo, FROM_DISPLAYID_13_DATA_BLOCK);
-    
+    if (p861info->valid.hdr_static_metadata != 0)
+    {
+        parseCta861HdrStaticMetadataDataBlock(p861info, pInfo, FROM_DISPLAYID_13_DATA_BLOCK);
+    }
+
     // base video
     parse861bShortTiming(p861info, pInfo, FROM_DISPLAYID_13_DATA_BLOCK);
     // yuv420-only video
     parse861bShortYuv420Timing(p861info, pInfo, FROM_DISPLAYID_13_DATA_BLOCK);
     // CEA861-F at 7.5.12 section about VFPDB block.
-    if (p861info->total_vfpdb != 0)
+    if (p861info->total_svr != 0)
     {
-        parse861bShortPreferredTiming(p861info, pInfo, FROM_DISPLAYID_13_DATA_BLOCK);
+        parseCta861NativeOrPreferredTiming(p861info, pInfo, FROM_DISPLAYID_13_DATA_BLOCK);
     }
 
     return NVT_STATUS_SUCCESS;
@@ -1387,8 +1322,7 @@ static NVT_STATUS parseDisplayIdDisplayInterfaceFeatures(NvU8 * block, NVT_DISPL
     DISPLAYID_INTERFACE_FEATURES_DATA_BLOCK * blk = (DISPLAYID_INTERFACE_FEATURES_DATA_BLOCK *)block;
     if (blk->header.data_bytes > DISPLAYID_INTERFACE_FEATURES_DATA_BLOCK_MAX_LEN)
     {
-        // Assert since this error is ignored
-        nvt_assert(0);
+
         return NVT_STATUS_ERR;
     }
 
@@ -1422,7 +1356,7 @@ static NVT_STATUS parseDisplayIdDisplayInterfaceFeatures(NvU8 * block, NVT_DISPL
 
     // Minimum Pixel Rate at Which YCbCr 4:2:0 Encoding Is Supported
     pInfo->u4.display_interface_features.minimum_pixel_rate_ycbcr420 = blk->minimum_pixel_rate_ycbcr420;
-    
+
     // Audio capability
     pInfo->u4.display_interface_features.audio_capability.support_32khz = DRF_VAL(T_DISPLAYID, _INTERFACE_FEATURES, _AUDIO_SUPPORTED_32KHZ, blk->supported_audio_capability);
     pInfo->u4.display_interface_features.audio_capability.support_44_1khz = DRF_VAL(T_DISPLAYID, _INTERFACE_FEATURES, _AUDIO_SUPPORTED_44_1KHZ, blk->supported_audio_capability);
