@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2019-2020 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2019-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -281,6 +281,132 @@ nvlink_lib_powerdown_links_from_active_to_L2_end:
     return status;
 }
 
+static NvlStatus
+_nvlink_lib_unilateral_powerdown_links_from_active_to_off
+(
+    nvlink_link **links,
+    NvU32         numLinks,
+    NvU32         flags
+)
+{
+    NvlStatus status = NVL_SUCCESS;
+    nvlink_link **ppTargetLinks = NULL;
+    NvU32 numTargetLinks;
+    NvU32 i;
+
+    if ((links == NULL) || (numLinks == 0))
+    {
+        NVLINK_PRINT((DBG_MODULE_NVLINK_CORE, NVLINK_DBG_LEVEL_ERRORS,
+            "%s: No links to shutdown\n",
+            __FUNCTION__));
+
+        return NVL_ERR_GENERIC;
+    }
+
+    ppTargetLinks = (nvlink_link **)nvlink_malloc( sizeof(nvlink_link *) * (numLinks));
+    if (ppTargetLinks == NULL)
+    {
+        return NVL_NO_MEM;
+    }
+
+    // Acquire the top-level lock
+    status = nvlink_lib_top_lock_acquire();
+    if (status != NVL_SUCCESS)
+    {
+        NVLINK_PRINT((DBG_MODULE_NVLINK_CORE, NVLINK_DBG_LEVEL_ERRORS,
+            "%s: Failed to acquire top-level lock\n",
+            __FUNCTION__));
+
+        if (ppTargetLinks != NULL)
+            nvlink_free((void *)ppTargetLinks);
+
+        return NVL_ERR_GENERIC;
+    }
+
+    //
+    // Top-level lock is now acquired. Proceed to traversing the device
+    // and link lists
+    //
+
+    // Double check whether we can unilaterally shutdown links
+    if (!nvlink_core_link_states_symmetric(links[0]))
+    {
+        // Release the top-level lock
+        nvlink_lib_top_lock_release();
+
+        if (ppTargetLinks != NULL)
+            nvlink_free((void *)ppTargetLinks);
+
+        // We got here because no connections were found
+        NVLINK_PRINT((DBG_MODULE_NVLINK_CORE, NVLINK_DBG_LEVEL_ERRORS,
+            "%s: No conns were found\n", __FUNCTION__));
+
+        return NVL_NOT_FOUND;
+    }
+
+    // Acquire the per-link locks for all links captured
+    status = nvlink_lib_link_locks_acquire(links, numLinks);
+    if (status != NVL_SUCCESS)
+    {
+        NVLINK_PRINT((DBG_MODULE_NVLINK_CORE, NVLINK_DBG_LEVEL_ERRORS,
+            "%s: Failed to acquire per-link locks\n",
+            __FUNCTION__));
+
+        // Release the top-level lock
+        nvlink_lib_top_lock_release();
+
+        if (ppTargetLinks != NULL)
+            nvlink_free((void *)ppTargetLinks);
+
+        return NVL_ERR_GENERIC;
+    }
+
+    // Sanity checking if the link is already in OFF/RESET state
+    numTargetLinks = 0;
+    for (i = 0; i < numLinks; i++)
+    {
+        // Check if link is in L2
+        if (nvlink_core_check_link_state(links[i], NVLINK_LINKSTATE_SLEEP))
+            continue;
+
+        // Check if link is OFF
+        if (nvlink_core_check_link_state(links[i], NVLINK_LINKSTATE_OFF))
+            continue;
+
+        // Check if link is in RESET
+        if (nvlink_core_check_link_state(links[i], NVLINK_LINKSTATE_RESET))
+            continue;
+
+        ppTargetLinks[numTargetLinks++] = links[i];
+    }
+
+    //
+    // All the required per-link locks are successfully acquired
+    // The connection list traversal is also complete now
+    // Release the top level-lock
+    //
+    nvlink_lib_top_lock_release();
+
+    if (numTargetLinks > 0)
+    {
+        //
+        // Squash status. If any side of link doesnt respond the link is
+        // shutdown unilaterally
+        //
+        (void)nvlink_core_unilateral_powerdown_links_from_active_to_off(ppTargetLinks,
+                                                                        numTargetLinks,
+                                                                        flags);
+    }
+
+    // Release the per-link locks
+    nvlink_lib_link_locks_release(links, numLinks);
+
+    if (ppTargetLinks != NULL)
+        nvlink_free((void *)ppTargetLinks);
+
+    return NVL_SUCCESS;
+}
+
 /**
  * [PSEUDO-CLEAN SHUTDOWN]
  *
@@ -304,10 +430,10 @@ nvlink_lib_powerdown_links_from_active_to_off
     nvlink_intranode_conn **conns    = NULL;
     nvlink_intranode_conn  *conn     = NULL;
     NvU32                   numConns = 0;
-    NvU32                   i;
+    NvU32                   i,j;
     NvU32                   lockLinkCount = 0;
     nvlink_link           **lockLinks = NULL;
-
+    NvBool                  bIsAlreadyPresent = NV_FALSE;
 
     if ((links == NULL) || (numLinks == 0))
     {
@@ -378,6 +504,24 @@ nvlink_lib_powerdown_links_from_active_to_off
         lockLinkCount++;
     }
 
+    if (lockLinkCount == 0)
+    {
+        if (conns != NULL)
+            nvlink_free((void *)conns);
+
+        if (lockLinks != NULL)
+            nvlink_free((void *)lockLinks);
+
+         // Release the top-level lock
+        nvlink_lib_top_lock_release();
+
+        //
+        // See if there are any disconnected links which support unilateral
+        // shutdown, and shut down those links if possible
+        //
+        return _nvlink_lib_unilateral_powerdown_links_from_active_to_off(links, numLinks, flags);
+    }
+
     // Acquire the per-link locks for all links captured
     status = nvlink_lib_link_locks_acquire(lockLinks, lockLinkCount);
     if (status != NVL_SUCCESS)
@@ -424,8 +568,51 @@ nvlink_lib_powerdown_links_from_active_to_off
             continue;
         }
 
-        conns[numConns] = conn;
-        numConns++;
+        //
+        // If device is using ALI based link training, it is possible
+        // for links to be still transitioning to active when a request to shutdown
+        // is made. Ensure that all connections transiton successfully to HS or fault
+        // before continuining to shutdown
+        //
+        if(links[0]->dev->enableALI)
+        {
+
+            status = nvlink_core_check_intranode_conn_state(conn, NVLINK_LINKSTATE_ACTIVE_PENDING);
+
+            if (status == NVL_SUCCESS)
+            {
+                status = nvlink_core_poll_link_state(conn->end0,
+                                             NVLINK_LINKSTATE_HS,
+                                             NVLINK_TRANSITION_ACTIVE_PENDING);
+
+                if (status != NVL_SUCCESS &&
+                    nvlink_core_check_intranode_conn_state(conn, NVLINK_LINKSTATE_FAULT) != NVL_SUCCESS)
+                {
+                    NVLINK_PRINT((DBG_MODULE_NVLINK_CORE, NVLINK_DBG_LEVEL_ERRORS,
+                        "%s: Connection between %s: %s and %s: %s is not ready for shutdown (link state is no in HS or FAULT). Soldiering on...\n",
+                        __FUNCTION__, conn->end0->dev->deviceName, conn->end0->linkName,
+                        conn->end1->dev->deviceName, conn->end1->linkName));
+                }
+            }
+        }
+
+        bIsAlreadyPresent = NV_FALSE;
+        // Check if the the connection is already included in the list
+        for (j = 0; j < numConns; j++)
+        {
+            if (conns[j] == conn)
+            {
+                bIsAlreadyPresent = NV_TRUE;
+                break;
+            }
+        }
+
+        // If this is a new connection, add it to the list
+        if (!bIsAlreadyPresent)
+        {
+            conns[numConns] = conn;
+            numConns++;
+        }
     }
 
     //
@@ -775,3 +962,108 @@ nvlink_lib_reset_links_end:
 
     return status;
 }
+
+
+NvlStatus
+nvlink_lib_powerdown_floorswept_links_to_off
+(
+    nvlink_device *dev
+)
+{
+    NvlStatus                status    = NVL_SUCCESS;
+    nvlink_link             *link      = NULL;
+    nvlink_link            **lockLinks = NULL;
+    NvU32                    lockLinkCount = 0;
+
+    if (dev == NULL)
+    {
+        NVLINK_PRINT((DBG_MODULE_NVLINK_CORE, NVLINK_DBG_LEVEL_ERRORS,
+            "%s: Bad device pointer specified.\n",
+            __FUNCTION__));
+
+        return NVL_ERR_GENERIC;
+    }
+
+    lockLinks = (nvlink_link **)nvlink_malloc(
+                            sizeof(nvlink_link *) * NVLINK_MAX_SYSTEM_LINK_NUM);
+    if (lockLinks == NULL)
+    {
+        return NVL_NO_MEM;
+    }
+
+    // Acquire the top-level lock
+    status = nvlink_lib_top_lock_acquire();
+    if (status != NVL_SUCCESS)
+    {
+        NVLINK_PRINT((DBG_MODULE_NVLINK_CORE, NVLINK_DBG_LEVEL_ERRORS,
+            "%s: Failed to acquire top-level lock\n",
+            __FUNCTION__));
+        goto nvlink_core_powerdown_floorswept_conns_to_off_end;
+    }
+
+    //
+    // If the device has less than or equal links in the IP then
+    // can be active, then skip floorsweeping
+    //
+    if (dev->numActiveLinksPerIoctrl >= dev->numLinksPerIoctrl)
+    {
+        nvlink_lib_top_lock_release();
+        goto nvlink_core_powerdown_floorswept_conns_to_off_end;
+    }
+
+    //
+    // Top-level lock is now acquired. Proceed to traversing the device
+    // and link lists and connection lists
+    //
+
+    // Get the array of link endpoints whose lock needs to be acquired
+    FOR_EACH_LINK_REGISTERED(link, dev, node)
+    {
+        if(link == NULL)
+        {
+            continue;
+        }
+
+        lockLinks[lockLinkCount] = link;
+        lockLinkCount++;
+    }
+
+    // Acquire the per-link locks for all links captured
+    status = nvlink_lib_link_locks_acquire(lockLinks, lockLinkCount);
+    if (status != NVL_SUCCESS)
+    {
+        NVLINK_PRINT((DBG_MODULE_NVLINK_CORE, NVLINK_DBG_LEVEL_ERRORS,
+            "%s: Failed to acquire per-link locks\n",
+            __FUNCTION__));
+
+        // Release the top-level lock
+        nvlink_lib_top_lock_release();
+
+        goto nvlink_core_powerdown_floorswept_conns_to_off_end;
+    }
+
+    //
+    // All the required per-link locks are successfully acquired
+    // The connection list traversal is also complete now
+    // Release the top level-lock
+    //
+    nvlink_lib_top_lock_release();
+
+    status = nvlink_core_powerdown_floorswept_conns_to_off(lockLinks, lockLinkCount, dev->numIoctrls,
+                        dev->numLinksPerIoctrl, dev->numActiveLinksPerIoctrl);
+
+    if (status == NVL_BAD_ARGS)
+    {
+        NVLINK_PRINT((DBG_MODULE_NVLINK_CORE, NVLINK_DBG_LEVEL_INFO,
+            "%s: Bad args passed in for floorsweeping. Chip might not support the feature\n",
+            __FUNCTION__));
+    }
+
+nvlink_core_powerdown_floorswept_conns_to_off_end:
+    if (lockLinks != NULL)
+    {
+        nvlink_free((void *)lockLinks);
+    }
+
+    return status;
+}
