@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2019-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2019 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -212,8 +212,15 @@ _inforom_nvlink_start_correctable_error_recording
 
     pNvlinkState->bCallbackPending = NV_FALSE;
 
-    nvswitch_task_create(device, &_nvswitch_nvlink_1hz_callback,
-                         NVSWITCH_INTERVAL_1SEC_IN_NS, 0);
+    if (!nvswitch_is_tnvl_mode_enabled(device))
+    {
+        nvswitch_task_create(device, &_nvswitch_nvlink_1hz_callback,
+                             NVSWITCH_INTERVAL_1SEC_IN_NS, 0);
+    }
+    else
+    {
+        NVSWITCH_PRINT(device, INFO, "Skipping NVLINK heartbeat task when TNVL is enabled\n");
+    }
 }
 
 NvlStatus
