@@ -872,6 +872,17 @@ nvswitch_init_soe_lr10
     return status;
 }
 
+void
+nvswitch_soe_init_l2_state_lr10
+(
+    nvswitch_device *device
+)
+{
+    NVSWITCH_PRINT(device, WARN,
+        "%s: Function not implemented on lr10\n",
+        __FUNCTION__);
+}
+
 /**
  * @brief  SOE construct
  *
@@ -1584,7 +1595,7 @@ _soeDmaStartTest
     nvswitch_os_memset(&cmd, 0, sizeof(cmd));
 
     cmd.hdr.unitId = RM_SOE_UNIT_CORE;
-    cmd.hdr.size   = sizeof(cmd);
+    cmd.hdr.size   = RM_SOE_CMD_SIZE(CORE, DMA_TEST);
 
     pDmaCmd = &cmd.cmd.core.dma_test;
     RM_FLCN_U64_PACK(&pDmaCmd->dmaHandle, &dmaHandle);
@@ -2064,7 +2075,7 @@ _soeForceThermalSlowdown_LR10
 
     nvswitch_os_memset(&cmd, 0, sizeof(cmd));
     cmd.hdr.unitId = RM_SOE_UNIT_THERM;
-    cmd.hdr.size = sizeof(cmd);
+    cmd.hdr.size = RM_SOE_CMD_SIZE(THERM, FORCE_SLOWDOWN);
     cmd.cmd.therm.cmdType = RM_SOE_THERM_FORCE_SLOWDOWN;
     cmd.cmd.therm.slowdown.slowdown = slowdown;
     cmd.cmd.therm.slowdown.periodUs = periodUs;
@@ -2541,6 +2552,18 @@ _soeWaitForInitAck_LR10
     return NV_OK;
 }
 
+NvlStatus
+nvswitch_soe_issue_ingress_stop_lr10
+(
+    nvswitch_device *device,
+    NvU32 nport,
+    NvBool bStop
+)
+{
+    // Not supported on LR10
+    return NVL_SUCCESS;
+}
+
 /*!
  * @brief   set hal function pointers for functions defined in LR10 (i.e. this file)
  *
