@@ -62,6 +62,6 @@ NvBool rmapiutilIsExternalClassIdInternalOnly(NvU32 externalClassId);
 NV_STATUS rmapiutilGetControlInfo(NvU32 cmd, NvU32 *pFlags,
                                   NvU32 *pAccessRight, NvU32 *pParamsSize);
 
-NvBool rmapiutilSkipErrorMessageForUnsupportedVgpuGuestControl(NvU32 cmd);
+NvBool rmapiutilSkipErrorMessageForUnsupportedVgpuGuestControl(OBJGPU *pGpu, NvU32 cmd);
 
 #endif /* RMAPI_UTILS_H */
