@@ -516,7 +516,8 @@ kceMapPceLceForNvlinkPeers_GB100
                 KernelCE *pKCeLce          = GPU_GET_KCE(pGpu, lceIndex);
                 if(pKCeLce != NULL)
                 {
-                   pKCeLce->ceCapsMask    |= NVBIT32(CE_CAPS_NVLINK_P2P);
+                   pKCeLce->ceCapsMask     |= NVBIT32(CE_CAPS_NVLINK_P2P);
+                   pKCeLce->nvlinkPeerMask |= NVBIT32(pRemoteGpu->gpuInstance);
                 }
                 NV_PRINTF(LEVEL_INFO, "Nvlink peer CE Mapping for GPU%d <-> GPU%d -- PCE Index: %d -> LCE Index: %d\n",
                           pGpu->gpuInstance,
@@ -564,6 +565,7 @@ kceMapPceLceForGRCE_GB100
     NvU32         hshubIndex;
     NvU32         pceIndex;
     KernelCE     *pKCeIter;
+    NvU32         pceExcludeMaskForSharedGrce;
 
     KCE_ITER_SHIM_BEGIN(pGpu, pKCeIter)
     {
@@ -577,6 +579,28 @@ kceMapPceLceForGRCE_GB100
 
         hshubIndex = CE_GET_LOWEST_AVAILABLE_IDX(pKCeIter->shimConnectingHubMask);
 
+        pceExcludeMaskForSharedGrce = pKCeIter->decompPceMask;
+
+        if (gpuIsCCFeatureEnabled(pGpu))
+        {
+            NvU32 numPcesPerLce;
+            NvU32 numLces;
+            NvU32 supportedPceMask;
+            NvU32 supportedLceMask;
+            NvU32 pcesPerHshub;
+
+            kceGetPceConfigForLceType(pGpu,
+                                      pKCe,
+                                      NV2080_CTRL_CE_LCE_TYPE_PCIE,
+                                      &numPcesPerLce,
+                                      &numLces,
+                                      &supportedPceMask,
+                                      &supportedLceMask,
+                                      &pcesPerHshub);
+
+            pceExcludeMaskForSharedGrce = pceExcludeMaskForSharedGrce | supportedPceMask;
+        }
+
         // Some PCEs have not been assigned. Pick one and use that for GRCE.
         if ((pAvailablePceMaskForConnectingHub[hshubIndex] & ~(pKCeIter->decompPceMask)) != 0)
         {
@@ -602,7 +626,7 @@ kceMapPceLceForGRCE_GB100
 
             for (pceIndex = 0; pceIndex < size; ++pceIndex)
             {
-                if ((NVBIT32(pceIndex) & pKCeIter->decompPceMask) == 0)
+                if ((NVBIT32(pceIndex) & pceExcludeMaskForSharedGrce) == 0)
                 {
                     if (pKCeIter->pPceLceMap[pceIndex] != NV2080_CTRL_CE_UPDATE_PCE_LCE_MAPPINGS_INVALID_LCE)
                     {
@@ -1591,3 +1615,77 @@ kceAssignCeCaps_GB100
         RMCTRL_SET_CAP(pKCeCaps, NV2080_CTRL_CE_CAPS, _CE_NVLINK_P2P);
     }
 }
+
+/**
+ * @brief Returns the CEs marked for P2P based on PCE-LCE mapping algorithm
+ *
+ * @param[in]      pKCe               KernelCE pointer
+ * @param[in]      pGpu               OBJGPU pointer
+ * @param[in]      gpuMask            GPU mask
+ * @param[out]     pNvlinkP2PCeMask   Mask of LCEs marked for P2P
+ */
+NV_STATUS
+kceGetP2PCes_GB100
+(
+    KernelCE *pKCe,
+    OBJGPU   *pGpu,
+    NvU32     gpuMask,
+    NvU32    *pNvlinkP2PCeMask
+)
+{
+    NvU32          gpuCount       = gpumgrGetSubDeviceCount(gpuMask);
+    KernelNvlink  *pKernelNvlink  = GPU_GET_KERNEL_NVLINK(pGpu);
+    KernelCE      *pKCeLoop;
+    NvU32          remoteGpuMask  = 0;
+
+    if (pKernelNvlink == NULL)
+    {
+        return NV_WARN_NOTHING_TO_DO;
+    }
+
+    //
+    // Get the remote GPU mask. In the case of loopback mode, GPU count will be
+    // 1. So, current GPU will be the remote GPU as well. If the GPU count is 2,
+    // then in the case, figure out the remote GPU mask.
+    //
+    if (gpuCount == 1)
+    {
+        remoteGpuMask = gpuMask;
+    }
+    else
+    {
+        OBJGPU  *pRemoteGpu     = NULL;
+        NvU32    gpuInstance    = 0;
+
+        while ((pRemoteGpu = gpumgrGetNextGpu(gpuMask, &gpuInstance)) != NULL)
+        {
+            if (pRemoteGpu != pGpu)
+            {
+                remoteGpuMask = NVBIT32(gpuGetInstance(pRemoteGpu));
+                break;
+            }
+        }
+    }
+
+    KCE_ITER_ALL_BEGIN(pGpu, pKCeLoop, 0)
+    {
+        if (pKCeLoop->bStubbed)
+            continue;
+
+        if (kceIsCeNvlinkP2P_HAL(pGpu, pKCeLoop))
+        {
+            if ((pKCeLoop->nvlinkPeerMask & remoteGpuMask) > 0)
+            {
+                *pNvlinkP2PCeMask |= NVBIT32(pKCeLoop->publicID);
+            }
+        }
+    }
+    KCE_ITER_END
+
+    if (*pNvlinkP2PCeMask == 0)
+    {
+        return kceGetP2PCes_GH100(pKCe, pGpu, gpuMask, pNvlinkP2PCeMask);
+    }
+
+    return NV_OK;
+}
