@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2006-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2006-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -49,6 +49,7 @@
 
 #include "vgpu/rpc.h"
 #include "vgpu/vgpu_events.h"
+#include "nvdevid.h"
 
 //
 // statics
@@ -270,33 +271,21 @@ memmgrSetZbcReferenced
 )
 {
     RM_API *pRmApi = GPU_GET_PHYSICAL_RMAPI(pGpu);
-    NV2080_CTRL_INTERNAL_MEMSYS_SET_ZBC_REFERENCED_PARAMS params = {0};
-    RsClient *pClient;
-    Subdevice *pSubdevice;
-    NvHandle hSubdevice;
-    NvU32 subDevInst;
+    NV0080_CTRL_INTERNAL_MEMSYS_SET_ZBC_REFERENCED_PARAMS params = {0};
 
     // Allocations are RPCed to host, so they are counted there
     if (IS_VIRTUAL_WITHOUT_SRIOV(pGpu))
         return;
 
     params.bZbcSurfacesExist = bZbcSurfacesExist;
-
-    NV_ASSERT_OR_RETURN_VOID(
-        serverGetClientUnderLock(&g_resServ, hClient, &pClient) == NV_OK);
-
-    subDevInst = gpumgrGetSubDeviceInstanceFromGpu(pGpu);
-
-    NV_ASSERT_OR_RETURN_VOID(subdeviceGetByInstance(pClient, hDevice, subDevInst, &pSubdevice) == NV_OK);
-
-    hSubdevice = RES_GET_HANDLE(pSubdevice);
+    params.subdevInstance = gpumgrGetSubDeviceInstanceFromGpu(pGpu);
 
     NV_ASSERT_OK(
         pRmApi->Control(
             pRmApi,
             hClient,
-            hSubdevice,
-            NV2080_CTRL_CMD_INTERNAL_MEMSYS_SET_ZBC_REFERENCED,
+            hDevice,
+            NV0080_CTRL_CMD_INTERNAL_MEMSYS_SET_ZBC_REFERENCED,
             &params,
             sizeof(params)));
 }
@@ -552,15 +541,7 @@ memmgrFreeHal_GM107
     if (FLD_TEST_DRF(OS32, _ATTR2, _ZBC_SKIP_ZBCREFCOUNT, _NO, pFbAllocInfo->pageFormat->attr2) &&
         memmgrIsKind_HAL(pMemoryManager, FB_IS_KIND_ZBC, pFbAllocInfo->format))
     {
-        //
-        // For vGPU, ZBC is used by the guest application. So for vGPU use case zbcsurface can
-        // be 0. Hence this ASSERT is not relevant for the VGPU HOST.
-        // For the long term fix, we will have to save the flags that are set at the time of
-        // object allocation and then while Free we will have to add assert based on the flags.
-        //
-        if (!pGpu->getProperty(pGpu, PDB_PROP_GPU_IS_VIRTUALIZATION_MODE_HOST_VGPU))
-            NV_ASSERT(pMemoryManager->zbcSurfaces !=0 );
-
+        NV_ASSERT(pMemoryManager->zbcSurfaces !=0 );
         if (pMemoryManager->zbcSurfaces != 0)
         {
             pMemoryManager->zbcSurfaces--;
@@ -597,6 +578,7 @@ memmgrGetBAR1InfoForDevice_GM107
     OBJEHEAP      *pVASHeap;
     NV_RANGE       bar1VARange = NV_RANGE_EMPTY;
     RsClient      *pClient = RES_GET_CLIENT(pDevice);
+    KernelMIGManager *pKernelMIGManager = GPU_GET_KERNEL_MIG_MANAGER(pGpu);
 
     /*
      * For legacy vGPU and SRIOV heavy, get BAR1 information from vGPU plugin.
@@ -647,54 +629,60 @@ memmgrGetBAR1InfoForDevice_GM107
         bar1Info->bar1AvailSize = 0;
         bar1Info->bar1MaxContigAvailSize = 0;
 
-        if (!kbusIsStaticBar1Enabled(pGpu, pKernelBus))
+        if (pVASHeap != NULL)
         {
-            // normal non-static BAR1 case
-            if (pVASHeap != NULL)
+            pVASHeap->eheapInfoForRange(pVASHeap, bar1VARange, NULL, &largestFreeSize, NULL, &freeSize);
+
+            // In the case of static BAR1, this is the best-effort max from the dynamic region
+            bar1Info->bar1MaxContigAvailSize = (NvU32)(largestFreeSize / 1024);
+
+            if ((!kbusIsStaticBar1Enabled(pGpu, pKernelBus)) ||
+                ((pKernelMIGManager != NULL) && kmigmgrIsMIGMemPartitioningEnabled(pGpu, pKernelMIGManager) && !kmigmgrIsDeviceUsingDeviceProfiling(pGpu, pKernelMIGManager, pDevice) ))
             {
-                pVASHeap->eheapInfoForRange(pVASHeap, bar1VARange, NULL, &largestFreeSize, NULL, &freeSize);
+                // normal non-static BAR1 case
+                // If VGPU, then the partitionable range falls outside of static BAR1 range
                 bar1Info->bar1AvailSize = (NvU32)(freeSize / 1024);
-                bar1Info->bar1MaxContigAvailSize = (NvU32)(largestFreeSize / 1024);
             }
-        }
-        else
-        {
-            //
-            // Actual BAR1 usage isn't interesting in static BAR1 because all the
-            // client BAR1 is already mapped. Also BAR1 >= client FB size.
-            // So free BAR1 that can stil be used is reported as
-            // bar1Size - FB in-use size
-            //
-            NV2080_CTRL_FB_GET_INFO_V2_PARAMS fbInfoParams = {0};
-            Subdevice *pSubdevice;
-            RM_API *pRmApi = rmapiGetInterface(RMAPI_GPU_LOCK_INTERNAL);
+            else
+            {
+                //
+                // Actual BAR1 usage isn't interesting in static BAR1 because all the
+                // client BAR1 is already mapped. Also BAR1 >= client FB size.
+                // Get the real amount. Add back the bar1MapSize to account for the
+                // static mapping and subtract off how much of that is in use
+                // actual avail = eheap_freeSize + static_bar1_size - (FB in use)
+                //
+                // The last non-2MB aligned FB chunk will show up as consuming BAR1,
+                // but it doesn't. With this heuristic, we can't underflow or go over the
+                // BAR1 reported size.
+                //
+                NvU32 gfid;
+                NvU32 fbInUse;
+                Subdevice *pSubdevice;
 
-            NvU32 fbInUse;
+                NV_ASSERT_OK_OR_RETURN(vgpuGetCallingContextGfid(pGpu, &gfid));
 
-            NV_ASSERT_OK_OR_RETURN(
-                subdeviceGetByInstance(pClient,
-                                       RES_GET_HANDLE(pDevice),
-                                       gpumgrGetSubDeviceInstanceFromGpu(pGpu),
-                                       &pSubdevice));
+                NV2080_CTRL_FB_GET_INFO_V2_PARAMS fbInfoParams = {0};
 
-            fbInfoParams.fbInfoList[0].index = NV2080_CTRL_FB_INFO_INDEX_HEAP_SIZE;
-            fbInfoParams.fbInfoList[1].index = NV2080_CTRL_FB_INFO_INDEX_HEAP_FREE;
+                fbInfoParams.fbInfoList[0].index = NV2080_CTRL_FB_INFO_INDEX_HEAP_SIZE;
+                fbInfoParams.fbInfoList[1].index = NV2080_CTRL_FB_INFO_INDEX_HEAP_FREE;
+                fbInfoParams.fbInfoListSize = 2;
 
-            fbInfoParams.fbInfoListSize = 2;
+                NV_ASSERT_OK_OR_RETURN(
+                    subdeviceGetByInstance(pClient,
+                                           RES_GET_HANDLE(pDevice),
+                                           gpumgrGetSubDeviceInstanceFromGpu(pGpu),
+                                           &pSubdevice));
 
-            NV_ASSERT_OK_OR_RETURN(
-                pRmApi->Control(pRmApi, pClient->hClient, RES_GET_HANDLE(pSubdevice),
-                                NV2080_CTRL_CMD_FB_GET_INFO_V2,
-                                &fbInfoParams,
-                                sizeof(fbInfoParams)));
+                // subdevice retrieval for this function is a bit circuitous here
+                NV_ASSERT_OK_OR_RETURN(subdeviceCtrlCmdFbGetInfoV2(pSubdevice, &fbInfoParams));
 
-            fbInUse = fbInfoParams.fbInfoList[0].data -
-                      fbInfoParams.fbInfoList[1].data;
+                fbInUse = fbInfoParams.fbInfoList[0].data -
+                          fbInfoParams.fbInfoList[1].data;
 
-            bar1Info->bar1AvailSize = bar1Info->bar1Size - fbInUse;
+                bar1Info->bar1AvailSize = ((freeSize + pKernelBus->bar1[gfid].staticBar1.size) / 1024) - fbInUse;
 
-            // Bug 4087553: we're requeted to return 0 for now
-            bar1Info->bar1MaxContigAvailSize = 0;
+            }
         }
     }
     else
@@ -1019,7 +1007,7 @@ memmgrInitReservedMemory_GM107
     // Reserved memory located at bottom of FB, base this at start of FB
     else
     {
-        tmpAddr = pMemoryManager->heapStartOffset;
+        tmpAddr = 0;
         if (bRsvdRegionIsValid)
         {
             tmpAddr = NV_MAX(pMemoryManager->Ram.fbRegion[rsvdRegion].base, tmpAddr);
@@ -1241,16 +1229,14 @@ memmgrSetMemDescPageSize_GM107
 
     if (ADDR_SYSMEM == addrSpace)
     {
+        NvU64 sysmemPageSize = RMCFG_FEATURE_PLATFORM_UNIX ? osGetPageSize() : pMemoryManager->sysmemPageSize;
         RmPhysAddr physAddr = memdescGetPte(pMemDesc, addressTranslation, 0);
         switch (pageSizeAttr)
         {
-            case RM_ATTR_PAGE_SIZE_INVALID:
-                NV_PRINTF(LEVEL_ERROR, "invalid page size attr\n");
-                return NV_ERR_INVALID_ARGUMENT;
             case RM_ATTR_PAGE_SIZE_DEFAULT:
                 newPageSize = _memmgrGetOptimalSysmemPageSize(physAddr,
                         pMemDesc, kgmmuGetBigPageSize_HAL(pKernelGmmu),
-                        pMemoryManager->sysmemPageSize);
+                        sysmemPageSize);
                 break;
             case RM_ATTR_PAGE_SIZE_4KB:
                 newPageSize = RM_PAGE_SIZE;
@@ -1279,6 +1265,9 @@ memmgrSetMemDescPageSize_GM107
                 NV_ASSERT_OR_RETURN(0 == (physAddr & (RM_PAGE_SIZE_256G - 1)), NV_ERR_INVALID_OFFSET);
                 newPageSize = RM_PAGE_SIZE_256G;
                 break;
+            case RM_ATTR_PAGE_SIZE_INVALID:
+                NV_PRINTF(LEVEL_ERROR, "invalid page size attr\n");
+                return NV_ERR_INVALID_ARGUMENT;
         }
     }
     else if (ADDR_FBMEM == addrSpace)
@@ -1286,9 +1275,6 @@ memmgrSetMemDescPageSize_GM107
         RmPhysAddr physAddr = memdescGetPte(pMemDesc, addressTranslation, 0);
         switch (pageSizeAttr)
         {
-            case RM_ATTR_PAGE_SIZE_INVALID:
-                NV_PRINTF(LEVEL_ERROR, "invalid page size attr\n");
-                return NV_ERR_INVALID_ARGUMENT;
             case RM_ATTR_PAGE_SIZE_DEFAULT:
             {
                 NvBool bUseDefaultHugePagesize = NV_TRUE;
@@ -1339,6 +1325,9 @@ memmgrSetMemDescPageSize_GM107
                 NV_ASSERT_OR_RETURN(0 == (physAddr & (RM_PAGE_SIZE_256G - 1)), NV_ERR_INVALID_OFFSET);
                 newPageSize = RM_PAGE_SIZE_256G;
                 break;
+            case RM_ATTR_PAGE_SIZE_INVALID:
+                NV_PRINTF(LEVEL_ERROR, "invalid page size attr\n");
+                return NV_ERR_INVALID_ARGUMENT;
         }
     }
 
@@ -1408,15 +1397,37 @@ memmgrGetRsvdSizeForSr_GM107
     MemoryManager *pMemoryManager
 )
 {
+    //
+    // Temporary WAR to override WDDM S/R buffer for specific skus
+    // Bug 5327051
+    //
+    static const NvU16 gb20x_devid[] = { 0x2B8C };
+    NvU32  pciDeviceID = DRF_VAL(_PCI, _DEVID, _DEVICE, pGpu->idInfo.PCIDeviceID);
+    NvBool overrideFbsrRsvdBufferSize = NV_FALSE;
+
+    for (NvU32 i = 0; i < NV_ARRAY_ELEMENTS(gb20x_devid); i++)
+    {
+        if (pciDeviceID == gb20x_devid[i])
+        {
+            overrideFbsrRsvdBufferSize = NV_TRUE;
+            break;
+        }
+    }
+
     if (((pMemoryManager->Ram.fbTotalMemSizeMb >> 10) >= 31) || IS_GSP_CLIENT(pGpu))
     {
         //
         // We need to reserve more memory for S/R if
-        // 1. FB size is > 32GB  Bug Id: 2468357
+        // 1. FB size is >= 31GB  Bug Id: 2468357
         // 2. Or GSP is enabled  Bug Id: 4312881
         //
         return 512 * 1024 * 1024;
     }
+    else if (overrideFbsrRsvdBufferSize)
+    {
+        // Bug 5327051: WAR to override WDDM S/R buffer for specific skus
+        return 300 * 1024 * 1024;
+    }
     else
     {
         return 256 * 1024 * 1024;
@@ -1839,6 +1850,12 @@ memmgrCalcReservedFbSpaceHal_GM107
         *rsvdSlowSize = pMemoryManager->rsvdMemorySizeIncrement;
         *rsvdISOSize = 0;
     }
+    // Temporary workaround to increase the heap size for NVBUG 4997009
+    if(IsGB20XorBetter(pGpu) && (IS_VGPU_GSP_PLUGIN_OFFLOAD_ENABLED(pGpu) && !IS_VIRTUAL(pGpu)))
+    {
+        // increase by 150 MB
+        *rsvdSlowSize += (150 << 20);
+    }
 
     if (RMCFG_FEATURE_PLATFORM_WINDOWS && pMemoryManager->bBug2301372IncreaseRmReserveMemoryWar)
     {
@@ -2022,16 +2039,6 @@ memmgrPreInitReservedMemory_GM107
             pKernelBus->bar2[GPU_GFID_PF].instBlockBase, GF100_BUS_INSTANCEBLOCK_SIZE);
     }
 
-    if (gpuIsSelfHosted(pGpu) && !RMCFG_FEATURE_PLATFORM_GSP)
-    {
-        //
-        // Reserve space for the test buffer used in coherent link test
-        // that is run early when memory allocation is not ready yet.
-        //
-        pKernelBus->coherentLinkTestBufferBase = tmpAddr;
-        tmpAddr += BUS_COHERENT_LINK_TEST_BUFFER_SIZE;
-    }
-
     //
     // This has to be the very *last* thing in reserved memory as it
     // will may grow past the 1MB reserved memory window.  We cannot
