@@ -32,13 +32,14 @@
 #include "core/locks.h"
 #include "gpu/gpu_access.h"
 #include "gpu/gpu.h"
-#include "syncgpuboost.h"
+#include "gpu/power/syncgpuboost.h"
 #include "nvlimits.h"
 #include "gpu_mgr/gpu_mgr.h"
 #include "gpu/gsp/gsp_static_config.h"
 #include "vgpu/vgpu_events.h"
 #include "gpu/perf/kern_perf_gpuboostsync.h"
 #include "nvdevid.h"
+#include "platform/sli/sli.h"
 
 /*-----------------------Static Private Method Prototypes---------------------*/
 static NV_STATUS _gpuboostmgrApplyPolicyFilters(NV0000_SYNC_GPU_BOOST_GROUP_CONFIG *);
@@ -643,6 +644,8 @@ _gpuboostmgrApplyPolicyFilters(NV0000_SYNC_GPU_BOOST_GROUP_CONFIG *pBoostConfig)
     for (i = 1; i < pBoostConfig->gpuCount; i++)
     {
         pGpuItr = gpumgrGetGpuFromId(pBoostConfig->gpuIds[i]);
+        NV_ASSERT_OR_RETURN(NULL != pGpuItr, NV_ERR_OBJECT_NOT_FOUND);
+
         if (pGpuGrp == gpumgrGetGpuGrpFromGpu(pGpuItr))
         {
             continue;
@@ -665,6 +668,8 @@ _gpuboostmgrApplyPolicyFilters(NV0000_SYNC_GPU_BOOST_GROUP_CONFIG *pBoostConfig)
     for (i = 0; i < pBoostConfig->gpuCount; i++)
     {
         pGpuItr = gpumgrGetGpuFromId(pBoostConfig->gpuIds[i]);
+        NV_ASSERT_OR_RETURN(NULL != pGpuItr, NV_ERR_OBJECT_NOT_FOUND);
+
         if (IsUnlinkedSLIEnabled(pGpuItr))
         {
             continue;
@@ -722,6 +727,7 @@ _gpuboostmgrApplyPolicyFilters(NV0000_SYNC_GPU_BOOST_GROUP_CONFIG *pBoostConfig)
         NvU16 subDeviceItr    = 0;
 
         pGpuItr = gpumgrGetGpuFromId(pBoostConfig->gpuIds[i]);
+        NV_ASSERT_OR_RETURN(NULL != pGpuItr, NV_ERR_OBJECT_NOT_FOUND);
 
         // Extract values for the GPU in the iteration.
         {
